/*
 * Copyright (c) 2022 VMware, Inc. All Rights Reserved.
 */

const jwt = require('jsonwebtoken')

const {reqCtxt} = require("./request_context");

function setAuthContextForRequest(req, res, next){
    let authHeader = req.headers.authorization;
    let tokenString;
    let token;
    if (authHeader){
        tokenString = authHeader.substring(7, authHeader.length);
    }
    if (tokenString){
        token = jwt.decode(tokenString);
        reqCtxt.set("user", token?.acct || token?.username || token?.azp); //this will be username, example abc@vmware.com
        reqCtxt.set("orgId", token?.context_name) //csp orgid
        reqCtxt.set("principal", Object.freeze(Object.assign({}, token))); //setting the complete token in the req cntxt
    }
    let ariaSource = req.headers["x-aria-source"];
    if (ariaSource) {
        reqCtxt.set("aria-source", ariaSource)
    }
    let origin = req.headers.origin
    if (origin) {
        reqCtxt.set("origin", origin)
    }
    next();
}

module.exports.setAuthContextForRequest = setAuthContextForRequest;