/*
 * Copyright (c) 2022 VMware, Inc. All Rights Reserved.
 */

/*
 * This function provides cost/complexity estimator for the paginated Connection result conforming to
 * relay connection spec. See- https://relay.dev/graphql/connections.htm
 * This basically looks at a field that "Returns" aka has a Type of *Connection.
 * Then checks if that field has the @cost directive. If there is a cost directive this returns nothing.
 * This causes the directiveCostEstimator to come into play.
 *
 * If no cost directive is present, it uses the "first" argument to multiply the childComplexity for this type.
 * If not first is provided a default of 100 will be used.
 */
function connectionCostEstimator(options){

    let {
        type,
        field,
        node,
        args,
        childComplexity
    } = options;

    let isConnection = field?.type?.name?.endsWith("Connection");
    if (isConnection){
        let costDirective = field?.astNode?.directives?.find(directive => directive.name.value === "cost");
        if (costDirective){
            return;
        } else {
            let firstArg= args.first || 100;
            return firstArg * childComplexity;
        }
    }
    return;
}

module.exports = connectionCostEstimator;