/*
 * Copyright (c) 2022 VMware, Inc. All Rights Reserved.
 */

const {init, FFEvents} = require('csp-ff-node-server-sdk');
const {gatewayConfig, smDeployment} = require('./config');
const {log} = require("./logger");

// mock version of cspFFClient when in self-managed mode
let cspFFClient = {
    evaluate: () => true,
    once: () => true,
    on:() => true
};

if(!smDeployment){
    // load gateway config
    const config = gatewayConfig();
    cspFFClient = init(config.cspFFSSDKKey);

    cspFFClient.on(FFEvents.INIT_INVALID_API_KEY, (errorMessage) =>{
        log.error("Invalid CSP Feature Flag SDK key. PLease check the config");
        log.error(`Error message is ${errorMessage}`)
        process.exit(129);
    })

    cspFFClient.on(FFEvents.INIT_ERROR, (errorMessage) => {
        log.error(`There was an error initing CSP FF SDK, error is: ${errorMessage}`);
    })

    cspFFClient.once(FFEvents.INIT_READY, () => {
        log.info("CSP FF SDK successfully inited.");
    })
}

module.exports.cspFFClient = cspFFClient;
module.exports.FFEvents = FFEvents;