/*
 * Copyright (c) 2022 VMware, Inc. All Rights Reserved.
 */

const {gatewayConfig} = require('./config');
const { mapSchema, getDirective, MapperKind } = require('@graphql-tools/utils');
const config = gatewayConfig();
const enabledFeatureStabilityValues= config.enabledFeatureStabilityValues;

//This currently only looks at specific elements,  but can be extended to look at other
//schema elements.
module.exports.featureStabilityDirectiveTransformer  =  function featureStabilityDirectiveTransformer(schema, directiveName='featureStability') {
    function validateFS(schemaElement, ffsValue) {
        const ALPHA_MSG = "\n\n**NOTE:** This API/functionality is in Alpha, This functionality can be unstable and schema elements may change over future iterations.";
        const BETA_MSG = "\n\n**NOTE:** This API/functionality is in Beta, this functionality is a candidate for a stable API but we reserve the right to fix issues including modifying the schema if required.";

        if (enabledFeatureStabilityValues.includes(ffsValue)) {
            let currentFieldDescription = schemaElement.description;
            switch (ffsValue) {
                case "ALPHA" :
                    schemaElement.description = [currentFieldDescription, ALPHA_MSG].filter(Boolean).join("");
                    break;
                case "BETA" :
                    schemaElement.description = [currentFieldDescription, BETA_MSG].filter(Boolean).join("");
                    break;
            }
            return schemaElement;
        } else {
            return null;
        }
    }

    return  mapSchema(schema, {
        [MapperKind.ARGUMENT]: (argumentConfig) => {
            const fsDirective = getDirective(schema, argumentConfig, directiveName)?.[0];
            if (fsDirective){
                return validateFS(argumentConfig, fsDirective.value);
            }
            return argumentConfig;
        },
        [MapperKind.INPUT_OBJECT_TYPE]: (type) => {
            const fsDirective = getDirective(schema, type, directiveName)?.[0];
            if (fsDirective){
                return validateFS(type, fsDirective.value);
            }
            return type;
        },
        [MapperKind.INPUT_OBJECT_FIELD]: (fieldConfig) => {
            const fsDirective = getDirective(schema, fieldConfig, directiveName)?.[0];
            if (fsDirective){
                return validateFS(fieldConfig, fsDirective.value);
            }
            return fieldConfig;
        },
        [MapperKind.OBJECT_FIELD]: (fieldConfig) => {
            const fsDirective = getDirective(schema, fieldConfig, directiveName)?.[0];
            if (fsDirective){
                return validateFS(fieldConfig, fsDirective.value);
            }
            return fieldConfig;
        },
        [MapperKind.ENUM_VALUE]: (enumValueConfig) => {
            const fsDirective = getDirective(schema, enumValueConfig, directiveName)?.[0];
            if (fsDirective){
                return validateFS(enumValueConfig, fsDirective.value);
            }
            return enumValueConfig;
        }
    });
};
