/*
 * Copyright (c) 2022 VMware, Inc. All Rights Reserved.
 */

const {gatewayConfig} = require('./config');
const got = require('got');
const {log} = require("./logger");

// load gateway config
const config = gatewayConfig();

async function handleHealthCheck() {
    let status = "HEALTHY";
    let degradedServices = [];

    for (const endpoint of config.endpoints) {
        try {
            let url = new URL(endpoint.url);
            let endpointCheckURL = url.origin + "/actuator/health/liveness";
            let response = await got(endpointCheckURL, {
                retry: {
                    limit: 0
                }
            });
        } catch (e) {
            log.debug(`Health check for ${endpoint.name} failed url used was ${endpoint.url}`);
            if (endpoint.isCritical === true) {
                status = "UNAVAILABLE";
            } else if (status !== "UNAVAILABLE") { //if error and status is not already set as UNAVAIL set to DEGRADED.
                status = "DEGRADED";
            }
            degradedServices.push(endpoint.name);
        }
    }
    return {
        "status": status,
        "degradedServices": degradedServices
    };
}

module.exports = handleHealthCheck;
