/*
 * Copyright (c) 2022 VMware, Inc. All Rights Reserved.
 */

const { mapSchema, getDirective, MapperKind } = require('@graphql-tools/utils');

/**
 * This class filters out schema elements marked with the @internal directive.
 * Filters out specific elements currently but Can be expanded as needed to other schema elements.
 * And it does nothing other than filter them out.
 * This should be used to filter out private elements from the public endpoint.
 */
module.exports.internalDirectiveTransformer  =  function internalDirectiveTransformer(schema, directiveName='internal') {
    return  mapSchema(schema, {
        [MapperKind.ARGUMENT]: (argumentConfig) => {
            const internalDirective = getDirective(schema, argumentConfig, directiveName)?.[0];
            return internalDirective ? null : argumentConfig;
        },
        [MapperKind.INPUT_OBJECT_TYPE]: (type) => {
            const internalDirective = getDirective(schema, type, directiveName)?.[0];
            return internalDirective ? null : type;
        },
        [MapperKind.INPUT_OBJECT_FIELD]: (fieldConfig) => {
            const internalDirective = getDirective(schema, fieldConfig, directiveName)?.[0];
            return internalDirective ? null : fieldConfig;
        },
        [MapperKind.OBJECT_FIELD]: (fieldConfig) => {
            const internalDirective = getDirective(schema, fieldConfig, directiveName)?.[0];
            return internalDirective ? null : fieldConfig;
        },
        [MapperKind.ENUM_VALUE]: (enumValueConfig) => {
            const internalDirective = getDirective(schema, enumValueConfig, directiveName)?.[0];
            return internalDirective ? null : enumValueConfig;
        }
    });
};
