/*
 * Copyright (c) 2022 VMware, Inc. All Rights Reserved.
 */

const winston = require("winston");
const expressWinston = require('express-winston');
const {reqCtxt} = require("./request_context");

module.exports.log = log = winston.createLogger({
    level: 'debug',
    format: winston.format.combine(
        winston.format.timestamp({
            format: 'YYYY-MM-DD HH:mm:ss.SSS'
        }),
        winston.format.printf(info => logFormat(info))
    ),
    defaultMeta: { },
    transports: [
        new winston.transports.Console()
    ]
});

function logFormat(info) {
    let trace_id = info.trace_id;
    let span_id = info.span_id;
    let parent_id = info.parent_span_id;
    return `${info.timestamp} ${info.level}: trace='${trace_id || "-"}',parent='${parent_id || "-"}',span='${span_id || "-"}',user='${reqCtxt.get("user") || "-"}',org='${reqCtxt.get("orgId") || "-"}',source=${reqCtxt.get("aria-source") || "-"},origin=${reqCtxt.get("origin") || "-"} ${info.message}`;
}

module.exports.requestLogger = function() {
    return expressWinston.logger({
        winstonInstance: log,
        requestWhitelist: ['url', 'method', 'query', 'body'],
        msg: (req, res) => {
            const complexity = reqCtxt.get("queryComplexity");
            return `HTTP ${req.method} ${req.url} ${res.statusCode} ${res.responseTime}ms qc:${complexity} ` + JSON.stringify(req.body, (k, v) => "authorization" === k.toLowerCase() ? "xxxxxxxxxxxxxxxxxxx" : v);
        }
    });
}
