/*
 * Copyright (c) 2023 VMware, Inc. All Rights Reserved.
 */
const {getComplexity, directiveEstimator, simpleEstimator} = require('graphql-query-complexity');
const connectionCostEstimator = require("./connection_cost_estimator");
const {reqCtxt} = require("./request_context");
const {log} = require("./logger");

//Currently setting an arbitrary high number to observe current query costs.
const MAX_PERMITTED_COST=process.env.MAX_COST_PERMITTED || Number.MAX_SAFE_INTEGER ;

const  estimators = [
    connectionCostEstimator,
    directiveEstimator({ name : "cost"}),

    simpleEstimator({
            defaultComplexity: 1
        }
    )
];

function checkQueryComplexity(validationContext, executionArgs) {
    return {
        OperationDefinition: (node) => {
            try {
                const complexity = getComplexity({
                    estimators,
                    schema : executionArgs.schema,
                    query : executionArgs.document,
                    variables : executionArgs.variableValues
                });

                reqCtxt.set("queryComplexity", complexity);
                if (complexity > MAX_PERMITTED_COST){
                    throw new Error(`Query is too complex: ${actual}. Maximum allowed complexity: ${max}`);
                }
            } catch (e) {
                log.error('Could not calculate complexity', e.message);
            }
        }
    }
}

module.exports = checkQueryComplexity;